searchData={"items":[{"type":"module","title":"Grammar","doc":"This module exposes functions and macros to create parsers of structured inputs. Parsers are defined as LL(1) grammars.\n\nA grammar must be LL(1), i.e. must be unambiguous and have a single token lookahead.\n\nOne can create parsers at runtime, using the `Grammar` functions, or define parsers at compile time using the `Grammar` macros.\n\nThe grammar is defined by a set of rules, each rule being a set of clauses. Clauses must be understood as\ndisjoinded paths in the rule resolution, or as the `or` operator in the classic notation.\n\nThe tokenization process relies on the [TokenExtractor](`Grammar.Tokenizer.TokenExtractor`) protocol, which is used to extract tokens from the input string.\nThis protocol is implemented for BitString and Regex, and can be extended to custom token types.","ref":"Grammar.html"},{"type":"module","title":"Spaces and line breaks handling - Grammar","doc":"By default, the tokenizer will drop spaces and line breaks.\n\nIf you want to keep them, you can pass the `drop_spaces: false` option to the `use Grammar` macro.\n\nWhen using the Tokenizer directly, you must pass `false` as second parameter to `Tokenizer.new/2` to keep spaces and line breaks.\n\nIn this case, you are fully responsible for handling spaces and line breaks in your rules.\n\n# Using the API\n\nCreating a new grammar is done by calling `Grammar.new/0`, then adding clauses to it using `Grammar.add_clause/5`.\n\nOnce all the rules are defined, the grammar must be prepared using `Grammar.prepare/1`.\n\nThe grammar is then ready to be used for parsing, by calling `Grammar.start/2` with the starting rule name,\nand then `Grammar.loop/2` with the input string wrapped in a `Tokenizer`.\n\n#### Example\n\n    iex> g = Grammar.new()\n    ...> |> Grammar.add_clause(:start, [\"hello\", :what], fn [\"hello\", what] -> \"hello #{what} !\" end)\n    ...> |> Grammar.add_clause(:what, [~r/[a-zA-Z]+/], fn [ident] -> ident end)\n    ...> |> Grammar.prepare!()\n    ...> |> Grammar.start(:start)\n    iex> Grammar.loop(g, Grammar.Tokenizer.new(\"hello world\"))\n    {:ok, \"hello world !\"}\n\n# Using the DSL\n\nTo declare a parser module, just `use` the Grammar module in your module, and define your rules using the `rule/2` and `rule?/2` macro.\nThe newly defined module will expose a `parse/1` function that will parse the input string, and return a tuple with the\ntokenizer in its final state, and result.\n\nSee `rule/2` for a full example.","ref":"Grammar.html#module-spaces-and-line-breaks-handling"},{"type":"module","title":"Options - Grammar","doc":"- `drop_spaces: true` (default): if set to `false`, the tokenizer will not drop spaces and line breaks.\n\n#### Example\n\nIn the following `MyModuleKO` module, the `start` rule doesn't handle spaces and line breaks,\nso it will fail if the input contains them.\n\n    iex> defmodule MyModuleKO do\n    ...>   use Grammar, drop_spaces: false\n    ...>\n    ...>   # spaces and linebreaks not handled\n    ...>\n    ...>   rule start(\"hello\", \"world\") do\n    ...>     [_hello, _world] = params\n    ...>     \"hello world\"\n    ...>   end\n    ...> end\n    iex> MyModuleKO.parse(\"helloworld\")\n    {:ok, \"hello world\"}\n    iex> MyModuleKO.parse(\"hello world\")\n    {:error, {1, 6}, :no_token}\n\nBut in the `MyModuleOK` module, the `start` explicitly handles spaces and line breaks between \"hello\" and \"world\".\n\nAnd even more that rule definition requires at least one space between \"hello\" and \"world\", parsing fails if no space is found.\n\n    iex> defmodule MyModuleOK do\n    ...>   use Grammar, drop_spaces: false\n    ...>\n    ...>   # spaces and linebreaks not handled\n    ...>\n    ...>   rule start(\"hello\", ~r/[\\s]+/, \"world\") do\n    ...>     [_hello, _spaces, _world] = params\n    ...>     \"hello world\"\n    ...>   end\n    ...> end\n    iex> MyModuleOK.parse(\"helloworld\")\n    {:error, {1, 6}, :no_token}\n    iex> MyModuleOK.parse(~s/hello  \\t world/)\n    {:ok, \"hello world\"}","ref":"Grammar.html#module-options"},{"type":"function","title":"Grammar.add_clause/5","doc":"Add a clause to the grammar.\n\nA clause is defined by a name, a substitution, a function to execute when the clause is matched, and an epsilon flag.\n\nClauses sharing the same name are considered as clauses of a single rule.\n\nThe substitution is a list of terms which declares the steps to match the clause, from left to right, first to last.\n\nEach term can be either\n- a rule name, which is an atom,\n- a value for which there is a [TokenExtractor](`Grammar.Tokenizer.TokenExtractor`) protocol implementation.\n\n[TokenExtractor](`Grammar.Tokenizer.TokenExtractor`) implementations are provided for BitString and Regex, and can be extended to custom token types.\n\nThe function is a callback to execute when the clause is fully matched. It is given a list as parameter, each element\nof that list is the value produced by the substitution of each term.\n\nThe epsilon flag indicates if the rule is mandatory or not.\n\n> ⚠️ the epsilon flag is used only when the *first clause of a rule* is added, even if it is defaulted to `false`.","ref":"Grammar.html#add_clause/5"},{"type":"function","title":"Example - Grammar.add_clause/5","doc":"The *second* clause of `:start` is marked as epsilon, but it is ignored.\n\n    iex> g = Grammar.new()\n    ...> |> Grammar.add_clause(:start, [\"hello\"], fn [value] -> value end)\n    ...> |> Grammar.add_clause(:start, [\"world\"], fn [value] -> value end, true) # `true` is ignored !\n    ...> |> Grammar.prepare!()\n    ...> |> Grammar.start(:start)\n    ...>\n    iex> Grammar.loop(g, Grammar.Tokenizer.new(\"hello\"))\n    {:ok, \"hello\"}\n    ...>\n    iex> Grammar.loop(g, Grammar.Tokenizer.new(\"world\"))\n    {:ok, \"world\"}\n    ...>\n    iex> Grammar.loop(g, Grammar.Tokenizer.new(\"pouet\"))\n    {:error, {1, 1}, :no_clause_matched}\n\n\nThe *first* clause of `:start` is marked as epsilon !\n\n    iex> g = Grammar.new()\n    ...> |> Grammar.add_clause(:start, [\"hello\"], fn [value] -> value end, true) # `true` is used !\n    ...> |> Grammar.add_clause(:start, [\"world\"], fn [value] -> value end, false) # `false` is ignored !\n    ...> |> Grammar.prepare!()\n    ...> |> Grammar.start(:start)\n    ...>\n    iex> Grammar.loop(g, Grammar.Tokenizer.new(\"hello\"))\n    {:ok, \"hello\"}\n    ...>\n    iex> Grammar.loop(g, Grammar.Tokenizer.new(\"world\"))\n    {:ok, \"world\"}\n    ...>\n    iex> Grammar.loop(g, Grammar.Tokenizer.new(\"pouet\"))\n    {:ok, nil}","ref":"Grammar.html#add_clause/5-example"},{"type":"function","title":"Grammar.loop/2","doc":"Process the input tokenizer, until completion or error.","ref":"Grammar.html#loop/2"},{"type":"function","title":"Grammar.new/0","doc":"Create a new empty `Grammar`.","ref":"Grammar.html#new/0"},{"type":"function","title":"Grammar.prepare/1","doc":"Use this function after defining all the rules of the grammar to prepare the grammar for parsing.\n\nThis function will proceed to grammar validation, and returns errors if any are found.\n\n> ⚠️ A grammar that is not prepared cannot be used for parsing, so `step/2` and `loop/2` will behave unexpectedly.","ref":"Grammar.html#prepare/1"},{"type":"function","title":"Grammar.prepare!/1","doc":"Same as prepare/1 but raises a RuntimeError if an error is found.","ref":"Grammar.html#prepare!/1"},{"type":"macro","title":"Grammar.rule/2","doc":"Use this macro to define rules of your grammar.\n\n**The first rule defined will be the entry rule of the grammar**.\n\nCalls to this macro sharing the same name will be grouped together as they define the same rule,\neach call is a possible path in the rule resolution.\n\nLets name a single call to `rule` a clause.\nAll clauses must be disjointed, i.e. they must not share the same first token.\nThey can be understood as the `or` operator in a rule.\n\nEach rule of rule clause is defined by\n- a name, which is an atom\n- a definition, which is a list of atoms or token prototypes\n- a block, which is the code to execute when the clause is fully matched\n\nWhen executed the code block is provided with a `params` binding, which is a list of the results of the clause steps.\n\nIn the case where a `rule` cannot be matched, a `RuntimeError` is raised (see `rule?/2` for a relaxed version).","ref":"Grammar.html#rule/2"},{"type":"macro","title":"Example - Grammar.rule/2","doc":"iex> defmodule NumberOfNameListParser do\n    ...>   use Grammar\n    ...>\n    ...>   rule start(\"[\", :list_or_empty_list) do\n    ...>     [_, list] = params\n    ...>     list || []\n    ...>   end\n    ...>\n    ...>   rule? list_or_empty_list(:item, :list_tail, \"]\") do\n    ...>     [item, list_tail, _] = params\n    ...>     [item | (list_tail || [])]\n    ...>   end\n    ...>\n    ...>   rule? list_tail(\",\", :item, :list_tail) do\n    ...>     [_, item, list_tail] = params\n    ...>     [item | (list_tail || [])]\n    ...>   end\n    ...>\n    ...>   rule item(~r/[0-9]+/) do\n    ...>     [number] = params\n    ...>     String.to_integer(number)\n    ...>   end\n    ...>\n    ...>   rule item(~r/[a-zA-Z]+/) do\n    ...>     [string] = params\n    ...>     string\n    ...>   end\n    ...> end\n    iex> GrammarTest.NumberOfNameListParser.parse(\"[1, toto, 23]\")\n    {:ok, [1, \"toto\", 23]}","ref":"Grammar.html#rule/2-example"},{"type":"macro","title":"Grammar.rule?/2","doc":"Same as `rule/2` but relaxed : if the rule cannot be matched, it will be valued as `nil`.\n\nUseful for optional or recursive rules.\n\nSee example in `rule/2`.","ref":"Grammar.html#rule?/2"},{"type":"function","title":"Grammar.start/2","doc":"Reset the grammar to its initial state, and set the starting rule name.\n\nUsually this function is called once before `loop/2`to process some input.\n\nOther usage may be to start at different rules within the same grammar, mainly for in dev testing.","ref":"Grammar.html#start/2"},{"type":"function","title":"Example - Grammar.start/2","doc":"iex> g = Grammar.new()\n    ...> |> Grammar.add_clause(:start, [\"<\", :lists?, \">\"], fn [_, ls, _] -> ls || [] end)\n    ...> |> Grammar.add_clause(:lists?, [:list, :lists?], fn [l, ls] -> [l | ls || []] end, true)\n    ...> |> Grammar.add_clause(:list, [\"[\", :elements?, \"]\"], fn [_, es, _] -> es || [] end)\n    ...> |> Grammar.add_clause(:elements?, [:element, :elements?], fn [e, es] -> [e | es || []] end, true)\n    ...> |> Grammar.add_clause(:element, [~r/[a-z]+/], fn [value] -> value end)\n    ...> |> Grammar.prepare!()\n    ...>\n    iex> g\n    ...> |> Grammar.start(:start)\n    ...> |> Grammar.loop(Grammar.Tokenizer.new(\"<[a b c] [dd ee ff]>\"))\n    {:ok, [[\"a\", \"b\", \"c\"], [\"dd\", \"ee\", \"ff\"]]}\n    ...>\n    iex> g = Grammar.start(g, :element)\n    ...> Grammar.loop(g, Grammar.Tokenizer.new(\"<[a b c] [dd ee ff]>\"))\n    {:error, {1, 1}, :no_clause_matched}\n    iex> Grammar.loop(g, Grammar.Tokenizer.new(\"ff\"))\n    {:ok, \"ff\"}\n    iex> g = Grammar.start(g, :list)\n    ...> Grammar.loop(g, Grammar.Tokenizer.new(\"<[a b c] [dd ee ff]>\"))\n    {:error, {1, 1}, :no_clause_matched}\n    iex> Grammar.loop(g, Grammar.Tokenizer.new(\"[a b c]\"))\n    {:ok, [\"a\", \"b\", \"c\"]}","ref":"Grammar.html#start/2-example"},{"type":"function","title":"Grammar.step/2","doc":"Step once in the input Tokenizer, using `grammar`.\n\nThis function returns a tuple which first term is either `:cont` or `:halt`.\n\n-  `:cont`: the second term is the updated `grammar` and the third term is the updated `tokenizer`.\n-  `:halt`: the second term is an error tuple, the third term is the position of the error in the input string, the fourth term is the `grammar` and the fifth term is the `tokenizer`.\n\nThis function is used internally by `loop/2`, though it can be used to manually step through the parsing process.","ref":"Grammar.html#step/2"},{"type":"function","title":"Example - Grammar.step/2","doc":"It takes 5 steps to reach the fist element of the input list.\n\n    iex> grammar = Grammar.new()\n    ...> |> Grammar.add_clause(:start, [:loop?], &Enum.at(&1, 0, []))\n    ...> |> Grammar.add_clause(:loop?, [:element, :loop?], fn [head, tail] -> [head | tail || []] end, true)\n    ...> |> Grammar.add_clause(:element, [:ident], &Enum.at(&1, 0))\n    ...> |> Grammar.add_clause(:ident, [~r/[a-z]+/], fn [ident] ->  ident end)\n    ...> |> Grammar.prepare!()\n    ...> |> Grammar.start(:start)\n    iex> tokenizer = Grammar.Tokenizer.new(\"a b c d\")\n    iex> {:cont, grammar, tokenizer} = Grammar.step(grammar, tokenizer)\n    iex> {:cont, grammar, tokenizer} = Grammar.step(grammar, tokenizer)\n    iex> {:cont, grammar, tokenizer} = Grammar.step(grammar, tokenizer)\n    iex> {:cont, grammar, tokenizer} = Grammar.step(grammar, tokenizer)\n    iex> {:cont, grammar, tokenizer} = Grammar.step(grammar, tokenizer)\n    iex> [{_callback, 1, [\"a\"]} | _] = grammar.heap","ref":"Grammar.html#step/2-example"},{"type":"type","title":"Grammar.error/0","doc":"","ref":"Grammar.html#t:error/0"},{"type":"type","title":"Grammar.error_type/0","doc":"","ref":"Grammar.html#t:error_type/0"},{"type":"type","title":"Grammar.first/0","doc":"","ref":"Grammar.html#t:first/0"},{"type":"type","title":"Grammar.rules/0","doc":"","ref":"Grammar.html#t:rules/0"},{"type":"type","title":"Grammar.t/0","doc":"","ref":"Grammar.html#t:t/0"},{"type":"module","title":"Grammar.Tokenizer","doc":"This module extracts the tokens from the input string.\n\nIt is driven by the parser to search for specific tokens only, when required.","ref":"Grammar.Tokenizer.html"},{"type":"function","title":"Grammar.Tokenizer.current_token/2","doc":"Returns the current token found in the input, if any.\nExpected tokens are passed as a list of token prototypes.\n\nThe token is not consumed, so two succesive calls to `current_token/2` will return the same token.","ref":"Grammar.Tokenizer.html#current_token/2"},{"type":"function","title":"Examples - Grammar.Tokenizer.current_token/2","doc":"iex> tokenizer = Grammar.Tokenizer.new(\"hello world\")\n    %Grammar.Tokenizer{\n      input: \"hello world\",\n      current_line: 1,\n      current_column: 1,\n      drop_spaces?: true\n    }\n    iex> {{\"hello\", {1, 1}}, _} = Grammar.Tokenizer.current_token(tokenizer, [\"hello\"])\n    iex> {{\"hello\", {1, 1}}, _} = Grammar.Tokenizer.current_token(tokenizer, [\"hello\"])\n    iex> {{nil, {1, 1}}, _} = Grammar.Tokenizer.current_token(tokenizer, [\"world\"])","ref":"Grammar.Tokenizer.html#current_token/2-examples"},{"type":"function","title":"Grammar.Tokenizer.new/2","doc":"Creates a new tokenizer for a given input.","ref":"Grammar.Tokenizer.html#new/2"},{"type":"function","title":"Parameters - Grammar.Tokenizer.new/2","doc":"- input: bitstring from which token will be extracted.\n  - drop_spaces?: when set to `false`, the tokenizer will not drop spaces and newlines.","ref":"Grammar.Tokenizer.html#new/2-parameters"},{"type":"function","title":"Examples - Grammar.Tokenizer.new/2","doc":"iex> Grammar.Tokenizer.new(\"this is the input\")\n    %Grammar.Tokenizer{\n      input: \"this is the input\",\n      current_line: 1,\n      current_column: 1,\n      drop_spaces?: true\n    }\n\n    iex> Grammar.Tokenizer.new(\"this is the input\", false)\n    %Grammar.Tokenizer{\n      input: \"this is the input\",\n      current_line: 1,\n      current_column: 1,\n      drop_spaces?: false\n    }","ref":"Grammar.Tokenizer.html#new/2-examples"},{"type":"function","title":"Grammar.Tokenizer.next_token/2","doc":"Returns the current token found in the input, and consumes it.\nThe expected token prototype is passed as second parameter.","ref":"Grammar.Tokenizer.html#next_token/2"},{"type":"function","title":"Examples - Grammar.Tokenizer.next_token/2","doc":"iex> tokenizer = Grammar.Tokenizer.new(\"hello world\")\n    iex> {{\"hello\", {1, 1}}, tokenizer} = Grammar.Tokenizer.next_token(tokenizer, \"hello\")\n    iex> {{\"world\", {1, 7}}, _} = Grammar.Tokenizer.next_token(tokenizer, \"world\")","ref":"Grammar.Tokenizer.html#next_token/2-examples"},{"type":"type","title":"Grammar.Tokenizer.t/0","doc":"","ref":"Grammar.Tokenizer.html#t:t/0"},{"type":"protocol","title":"Grammar.Tokenizer.TokenExtractor","doc":"This protocol exposes the functions needed to extract tokens from the input string.","ref":"Grammar.Tokenizer.TokenExtractor.html"},{"type":"function","title":"Grammar.Tokenizer.TokenExtractor.match?/2","doc":"Returns true if the token matches the token prototype.\n\nThis function is used orient the parser in the right clause of a rule, by comparing the current token\nto the clause list of first tokens.","ref":"Grammar.Tokenizer.TokenExtractor.html#match?/2"},{"type":"function","title":"Example - Grammar.Tokenizer.TokenExtractor.match?/2","doc":"iex> Grammar.Tokenizer.TokenExtractor.match?(\"hello\", \"hello\")\n    true\n\n    iex> Grammar.Tokenizer.TokenExtractor.match?(\"hello\", \"Horld\")\n    false\n\n    iex> Grammar.Tokenizer.TokenExtractor.match?(~r/[0-9]+/, \"013456\")\n    true\n\n    iex> Grammar.Tokenizer.TokenExtractor.match?(~r/[0-9]+/, \"a013456\")\n    false","ref":"Grammar.Tokenizer.TokenExtractor.html#match?/2-example"},{"type":"function","title":"Grammar.Tokenizer.TokenExtractor.try_read/2","doc":"Try to read a token from the input string.\n\nIf the token is found, returns the token and its length in the input string.\nReturns nil otherwise.","ref":"Grammar.Tokenizer.TokenExtractor.html#try_read/2"},{"type":"function","title":"Example - Grammar.Tokenizer.TokenExtractor.try_read/2","doc":"iex> Grammar.Tokenizer.TokenExtractor.try_read(\"hello\", \"hello world\")\n    {\"hello\", 5}\n\n    iex> Grammar.Tokenizer.TokenExtractor.try_read(\"Hello\", \"hello world\")\n    nil\n\n    iex> Grammar.Tokenizer.TokenExtractor.try_read(~r/[0-9]+/, \"013456toto\")\n    {\"013456\", 6}\n\n    iex> Grammar.Tokenizer.TokenExtractor.try_read(~r/[a-z]+/, \"013456toto\")\n    nil","ref":"Grammar.Tokenizer.TokenExtractor.html#try_read/2-example"},{"type":"type","title":"Grammar.Tokenizer.TokenExtractor.t/0","doc":"All the types that implement this protocol.","ref":"Grammar.Tokenizer.TokenExtractor.html#t:t/0"},{"type":"module","title":"Grammar.Tokenizer.TokenExtractorHelper","doc":"This module provides helper functions to work with TokenExtractor implementations using Regex.","ref":"Grammar.Tokenizer.TokenExtractorHelper.html"},{"type":"function","title":"Grammar.Tokenizer.TokenExtractorHelper.normalize_regex/1","doc":"Normalizes a regex to ensure it starts with `^`, while keeping the original options.","ref":"Grammar.Tokenizer.TokenExtractorHelper.html#normalize_regex/1"},{"type":"function","title":"Grammar.Tokenizer.TokenExtractorHelper.try_read_from_regex/2","doc":"Tries to read a token from the input string using a regex pattern.","ref":"Grammar.Tokenizer.TokenExtractorHelper.html#try_read_from_regex/2"},{"type":"module","title":"Grammar.Clause","doc":"A Clause entity represents a grammar clause, by its substitution and callback to execute.\n\nThe substitution is a list of steps, which can be either a rule name or a token prototype.\n\nThe parameter `function` is the callback to execute when the clause is fully matched. It is\ngiven a list as parameter, each element of that list is the value produced by the substitution\nof each term.","ref":"Grammar.Clause.html"},{"type":"function","title":"Grammar.Clause.new/2","doc":"Create a new clause with a given substitution and callback function.","ref":"Grammar.Clause.html#new/2"},{"type":"type","title":"Grammar.Clause.callback/0","doc":"","ref":"Grammar.Clause.html#t:callback/0"},{"type":"type","title":"Grammar.Clause.substitution/0","doc":"","ref":"Grammar.Clause.html#t:substitution/0"},{"type":"type","title":"Grammar.Clause.t/0","doc":"","ref":"Grammar.Clause.html#t:t/0"},{"type":"module","title":"Grammar.Rule","doc":"A `Rule` entity represents a single rule in the grammar.\n\nIt has a name, a list of mutually exclusive clauses\nand `epsilon` flag indicating if the rule is mandatory or not.","ref":"Grammar.Rule.html"},{"type":"function","title":"Grammar.Rule.add_clause/2","doc":"Append a clause to the rule.\n\nClauses must be mutually exclusive, and must added in the order of\ntheir priority if ambiguity during token extraction is possible (which must\nbe avoided as much as possible).","ref":"Grammar.Rule.html#add_clause/2"},{"type":"function","title":"Grammar.Rule.epsilon?/1","doc":"Return `true` is the rule is not mandatory.\n\nIf a `Rule` is optional, and so `epsilon` is true, it is reduced to `nil` during the parsing\nif not entered.","ref":"Grammar.Rule.html#epsilon?/1"},{"type":"function","title":"Grammar.Rule.new/2","doc":"Create a new empty rule with a given name and epsilon flag.","ref":"Grammar.Rule.html#new/2"},{"type":"type","title":"Grammar.Rule.name/0","doc":"","ref":"Grammar.Rule.html#t:name/0"},{"type":"type","title":"Grammar.Rule.t/0","doc":"","ref":"Grammar.Rule.html#t:t/0"},{"type":"module","title":"Grammar.RulesChecker","doc":"This module exports functions to check the correctness of the grammar rules.","ref":"Grammar.RulesChecker.html"},{"type":"function","title":"Grammar.RulesChecker.check_all_rules_exist/1","doc":"Check if all rules exist in the grammar, i.e. a rule referred to in a substitution list is\nactually defined.","ref":"Grammar.RulesChecker.html#check_all_rules_exist/1"},{"type":"function","title":"Grammar.RulesChecker.check_rules_are_non_left_recursive/1","doc":"Check if the rules are non-left-recursive.","ref":"Grammar.RulesChecker.html#check_rules_are_non_left_recursive/1"},{"type":"function","title":"Grammar.RulesChecker.check_rules_are_not_ambiguous/1","doc":"Check if the rules are not ambiguous, i.e. a given token drives only to one reduction.","ref":"Grammar.RulesChecker.html#check_rules_are_not_ambiguous/1"},{"type":"function","title":"Grammar.RulesChecker.stringify_error/2","doc":"Express an error returned by a checker function in a human-readable form.","ref":"Grammar.RulesChecker.html#stringify_error/2"},{"type":"type","title":"Grammar.RulesChecker.ambiguity/0","doc":"","ref":"Grammar.RulesChecker.html#t:ambiguity/0"},{"type":"type","title":"Grammar.RulesChecker.miss/0","doc":"","ref":"Grammar.RulesChecker.html#t:miss/0"},{"type":"type","title":"Grammar.RulesChecker.path/0","doc":"","ref":"Grammar.RulesChecker.html#t:path/0"},{"type":"module","title":"Grammar.CodeGen","doc":"This module exposes functions required to generate the code for the parser derived from grammar rules.\n\nIt uses `Clause` struct to store the definition of clauses. `Clause`s sharing the same `name` are\nconsidered as clauses of a single rule.","ref":"Grammar.CodeGen.html"},{"type":"function","title":"Grammar.CodeGen.build_grammar/2","doc":"Produces the quoted expression of the code that builds the grammar from the stored clauses.","ref":"Grammar.CodeGen.html#build_grammar/2"},{"type":"function","title":"Grammar.CodeGen.build_rule_body_functions/1","doc":"Produces the list of quoted expressions of callback functions, on for each clause.","ref":"Grammar.CodeGen.html#build_rule_body_functions/1"},{"type":"function","title":"Grammar.CodeGen.store_clause/5","doc":"Produces the quoted expression of the code that stores a clause definition in the current module.","ref":"Grammar.CodeGen.html#store_clause/5"},{"type":"module","title":"Grammar.CodeGen.Clause","doc":"Stores a clause definition.\n\n`Clauses` are used to produce code, so parameters `def` and `blk` must be quoted expressions.","ref":"Grammar.CodeGen.Clause.html"},{"type":"function","title":"Grammar.CodeGen.Clause.new/4","doc":"Create a new clause with a given name, definition, block and epsilon flag.","ref":"Grammar.CodeGen.Clause.html#new/4"},{"type":"type","title":"Grammar.CodeGen.Clause.t/0","doc":"","ref":"Grammar.CodeGen.Clause.html#t:t/0"},{"type":"extras","title":"Grammar","doc":"# Grammar\n\n*One rule to ring them all*\n\nEasily build parsers / transformers for structured data.","ref":"readme.html"},{"type":"extras","title":"Overview - Grammar","doc":"An Elixir library that helps building parsers / transformers for LL(1) structured data.","ref":"readme.html#overview"},{"type":"extras","title":"Usage - Grammar","doc":"> Please have a look at the examples for more advanced use cases","ref":"readme.html#usage"},{"type":"extras","title":"API - Grammar","doc":"The following grammar extracts a list of integers from string of comma separated number, and enclosed by square brackets.\n\n```elixir\n  Grammar.new()\n  |> Grammar.add_clause(:list, [\"[\", :content, \"]\"], fn [\"[\", list, \"]\"] -> list || [] end)\n  |> Grammar.add_clause(:content, [:number, :list_tail], fn [number, list] -> [number | list || []] end, true)\n  |> Grammar.add_clause(:list_tail, [\",\", :number, :list_tail], fn [\",\", number, list] -> [number | list || []] end, true)\n  |> Grammar.add_clause(:number, [~r/[0-9]+/], fn [number] -> String.to_integer(number) end)\n  |> Grammar.prepare!()\n  |> Grammar.start(:list)\n  |> Grammar.loop(Grammar.Tokenizer.new(\"[1, 2, 6, 12]\"))\n\n  {:ok, [1, 2, 6, 12]}\n```","ref":"readme.html#api"},{"type":"extras","title":"DSL - Grammar","doc":"`use Grammar` and write \"rule\" functions that have direct access to parsing results.\n\nYou are fully in control of what is returned by each rule function.\n\nUnder the hood, the `rule/2` macro uses the regular API to build a `Grammar` at compile time, and bury it in the module you're defining. \n\n#### Example\n\nThis is the same grammar parser as in the [API section](#api), but under the form of a dedicated module.\n\n```elixir\ndefmodule NumberListReader do\n  use Grammar\n\n  rule list(\"[\", :list_tail, \"]\") do\n    [_, list, _] = params\n    case list do\n      nil -> []\n      _ -> list\n    end\n  end\n\n  rule? list_tail(:number, :list_tail_tail) do\n    [number, list] = params\n    [number | (list || [])]\n  end\n\n  rule? list_tail_tail(\",\", :number, :list_tail_tail) do\n    [_, number, list] = params\n    [number | (list || [])]\n  end\n\n  rule number(~r/[0-9]+/) do\n    [number] = params\n    String.to_integer(number)\n  end\nend\n\n$iex> NumberListReader.parse(\"[1, 2, 6, 12]\")\n{:ok, [1, 2, 6, 12]}\n```","ref":"readme.html#dsl"},{"type":"extras","title":"MIT License","doc":"# MIT License\n\nCopyright (c) 2024 Nicolas Michel\n\nPermission is hereby granted, free of charge, to any person obtaining\na copy of this software and associated documentation files (the\n\"Software\"), to deal in the Software without restriction, including\nwithout limitation the rights to use, copy, modify, merge, publish,\ndistribute, sublicense, and/or sell copies of the Software, and to\npermit persons to whom the Software is furnished to do so, subject to\nthe following conditions:\n\nThe above copyright notice and this permission notice shall be\nincluded in all copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND,\nEXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF\nMERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND\nNONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE\nLIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION\nOF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION\nWITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\n## [0.3.0] - 2024-11-29","ref":"changelog.html"},{"type":"extras","title":"Added - Changelog","doc":"- `Grammar` now exposes a public API to build grammar processors at runtime.\n- Some livebooks.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Revamp code generation to use the new public API.\n- Stack depth during input processing is now bound to grammar size, not to input size.\n\n## [0.2.0] - 2024-11-11","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- This CHANGELOG.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Improve runtime performance.\n\n## [0.1.0] - 2024-11-08","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- First release.\n\n\n# Diff\n[0.3.0](https://github.com/nmichel/ex_grammar/releases/tag/v0.3.0)\n[0.1.0](https://github.com/nmichel/ex_grammar/releases/tag/v0.1.0)\n[0.2.0](https://github.com/nmichel/ex_grammar/releases/tag/v0.2.0)","ref":"changelog.html#added"},{"type":"extras","title":"Math expression to french","doc":"# Math expression to french\n\nThe module `MathToFrench` can read simple integer math expressions, like \"1+2\", \"(42-2)*12 / 5\",\nand rewrite them in ... french.\n\n```\niex> MathToFrench.parse(\"(42-2)*12 / 5\")\n{:ok, \"(42 moins 2) multiplié par 12 divisé par 5\"}\n```\n\nHere follows its declaration.\n\n```elixir\ndefmodule MathToFrench do\n  use Grammar\n\n  # This rule wille be the entry point.\n  # Note that we could get rid of it, as it add no real value to\n  # the output.\n  rule start(:expression) do\n    [program] = params\n    \"#{program}\"\n  end\n\n  # a math expression is a composed of \"terms\" joined by\n  # symbol + or -.\n  # This ensure precedence of other operators over the them.\n  rule expression(:term, :expression_cont) do\n    join_non_nil(params)\n  end\n\n  rule? expression_cont(\"+\", :term, :expression_cont) do\n    [_, term, cont] = params\n    join_non_nil([\"plus\", term, cont])\n  end\n\n  rule? expression_cont(\"-\", :term, :expression_cont) do\n    [_, term, cont] = params\n    join_non_nil([\"moins\", term, cont])\n  end\n\n  # A term is composed of elements joined by operators * or /.\n  # Those elements are reduced by rule `factor`.\n  rule term(:factor, :term_cont) do\n    join_non_nil(params)\n  end\n\n  rule? term_cont(\"*\", :factor, :term_cont) do\n    [_, factor, cont] = params\n    join_non_nil([\"multiplié par\", factor, cont])\n  end\n\n  rule? term_cont(\"/\", :factor, :term_cont) do\n    [_, factor, cont] = params\n    join_non_nil([\"divisé par\", factor, cont])\n  end\n\n  # Eventually a factor is either a integer number or\n  # a parenthesized expression, as in rule `expression`.\n  rule factor(:number) do\n    [number] = params\n    number\n  end\n\n  rule factor(\"(\", :expression, \")\") do\n    [_, expression, _] = params\n    \"(#{expression})\"\n  end\n\n  rule number(~r/[0-9]+/) do\n    [number] = params\n    number\n  end\n\n  defp join_non_nil(params) when is_list(params) do\n    params |> Enum.reject(&is_nil/1) |> Enum.join(\" \")\n  end\nend\n```","ref":"math_to_french.html"},{"type":"extras","title":"Math expression to function","doc":"# Math expression to function\n\n> ⚠️ Consider reading [Math to french](./math_to_french.md) example first.\n\nThe module `MathToFunction` can read simple integer math expressions too,\nbut this time the parse/1 function returns a function that produce the value of the expression when called.\n\n```\niex> {:ok, calcfn} = MathToFunction.parse(\"(42-2)*12 / 5\")\niex> calcfn.()\n96.0\n```\n\nHere follows its declaration, a little bit more chalenging.\n\n```elixir\ndefmodule MathToFunction do\n  use Grammar\n\n  rule expression(:term, :expression_cont) do\n    [term, cont] = params\n    fn -> (cont || & &1).(term.()) end\n  end\n\n  rule? expression_cont(\"+\", :term, :expression_cont) do\n    [_, term, cont] = params\n    fn left -> (cont || & &1).((left + term.())) end\n  end\n\n  rule? expression_cont(\"-\", :term, :expression_cont) do\n    [_, term, cont] = params\n    fn left -> (cont || & &1).((left - term.())) end\n  end\n\n  rule term(:factor, :term_cont) do\n    [factor, cont] = params\n    fn -> (cont || & &1).(factor.()) end\n  end\n\n  rule? term_cont(\"*\", :factor, :term_cont) do\n    [_, factor, cont] = params\n    fn left -> (cont || & &1).((left * factor.())) end\n  end\n\n  rule? term_cont(\"/\", :factor, :term_cont) do\n    [_, factor, cont] = params\n    fn left -> (cont || & &1).((left / factor.())) end\n  end\n\n  rule factor(:number) do\n    [number] = params\n    number\n  end\n\n  rule factor(\"(\", :expression, \")\") do\n    [_, expression, _] = params\n    expression\n  end\n\n  rule number(~r/[0-9]+/) do\n    [number] = params\n    number_value = String.to_integer(number)\n    fn -> number_value end\n  end\nend\n```","ref":"math_to_function.html"},{"type":"extras","title":"List of IP addresses","doc":"# List of IP addresses\n\nThis example illustrates the use of [TokenExtractor](`Grammar.Tokenizer.TokenExtractor`) protocol for user defined token.\n\nFirst, we define the token prototype to catch IP address. In our simple case, the field `ip` will contain the list of integer values that compose the address.\n\n```elixir\ndefmodule IP do\n  defstruct ip: nil\nend\n```\n\nThen we implement the [TokenExtractor](`Grammar.Tokenizer.TokenExtractor`) protocol for that new token type.\n\nHere we are providing a naive implementation, based on a (poor) regular expression and [TokenExtractorHelper](`Grammar.Tokenizer.TokenExtractorHelper`) helper module.\n\nOne is free to implement a more robust / efficient version, as long as it respects the protocol.\n\n```elixir\ndefimpl Grammar.Tokenizer.TokenExtractor, for: IP do\n  @pattern ~r/^\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}/\n\n  alias Grammar.Tokenizer.TokenExtractorHelper\n\n  def try_read(_token_prototype, input_string) do\n    TokenExtractorHelper.try_read_from_regex(@pattern, input_string)\n    |> case do\n      nil ->\n        nil\n\n      {extracted_string, length} ->\n        token =\n          extracted_string\n          |> String.split(\".\")\n          |> then(&struct(IP, ip: &1))\n\n        {token, length}\n    end\n  end\n\n  def match?(%IP{}, %IP{}), do: true\n  def match?(_prototype, _token), do: false\nend\n```\n\nThen use this new token to define rules.\n\n```elixir\ndefmodule IPListReader do\n  use Grammar\n\n  rule ips(%IP{}, :ips_tail) do\n    [ip, tail] = params\n    [ip | (tail || [])]\n  end\n\n  rule? ips_tail(\",\", %IP{}, :ips_tail) do\n    [_, ip, tail] = params\n    [ip | (tail || [])]\n  end\nend\n```\n\nAnd we are done !\n\n```elixir\n$iex> IPListReader.parse(\"\n   1.2.3.4 ,\n   3.2.3.4 ,\n   4.2.3.4\n\")\n\n{:ok,\n [\n   %IP{ip: [\"1\", \"2\", \"3\", \"4\"]},\n   %IP{ip: [\"3\", \"2\", \"3\", \"4\"]},\n   %IP{ip: [\"4\", \"2\", \"3\", \"4\"]}\n ]}\n```","ref":"ip_list.html"},{"type":"extras","title":"Intro to DSL : parse a comma separated list of identifiers","doc":"# Intro to DSL : parse a comma separated list of identifiers\n\n```elixir\nMix.install([:grammar])\n```","ref":"dsl_intro.html"},{"type":"extras","title":"Local setup - Intro to DSL : parse a comma separated list of identifiers","doc":"To use this livebook within a local clone the project, use the following setup, and launch the livebook from it's directory.\n\n<!-- livebook:{\"force_markdown\":true} -->\n\n```elixir\nMix.install(\n  [\n    {:grammar, path: Path.join(__DIR__, \"..\"), env: :dev}\n  ],\n  lockfile: :grammar\n)\n```","ref":"dsl_intro.html#local-setup"},{"type":"extras","title":"Section - Intro to DSL : parse a comma separated list of identifiers","doc":"```elixir\ndefmodule MyGrammar do\n  use Grammar\n\n  rule list(\"[\", :list?, \"]\") do\n    [_, list, _] = params\n    list || []\n  end\n\n  # A maybe empty\n  rule? list?(:element, :list_cont?) do\n    [element, list_cont] = params\n    [element | list_cont || []]\n  end\n\n  # An element of the list matches a peculiar regex\n  rule element(~r/[a-zA-Z]+/) do\n    [element] = params\n    element\n  end\n\n  # List continuation may be empty\n  rule? list_cont?(\",\", :element, :list_cont?) do\n    [_, element, list_cont] = params\n    [element | list_cont || []]\n  end\nend\n```","ref":"dsl_intro.html#section"},{"type":"extras","title":"Parse some valid inputs - Intro to DSL : parse a comma separated list of identifiers","doc":"```elixir\n# Empty list\nIO.inspect(MyGrammar.parse(\"[]\"))\n\n# Single item list\nIO.inspect(MyGrammar.parse(\"[a]\"))\n\n# Regular list\nIO.inspect(MyGrammar.parse(\"[a, bb, ccc]\"))\n```","ref":"dsl_intro.html#parse-some-valid-inputs"},{"type":"extras","title":"Try parse invalid inputs - Intro to DSL : parse a comma separated list of identifiers","doc":"```elixir\nMyGrammar.parse(\"ident\")\n```\n\n```elixir\nMyGrammar.parse(\"[ident\")\n```\n\n```elixir\nMyGrammar.parse(\"[ident,]\")\n```","ref":"dsl_intro.html#try-parse-invalid-inputs"},{"type":"extras","title":"Grammar to parse a comma separated list of numbers","doc":"# Grammar to parse a comma separated list of numbers\n\n```elixir\nMix.install([:grammar])\n```","ref":"grammar_intro.html"},{"type":"extras","title":"Local setup - Grammar to parse a comma separated list of numbers","doc":"To use this livebook within a local clone the project, use the following setup, and launch the livebook from it's directory.\n\n<!-- livebook:{\"force_markdown\":true} -->\n\n```elixir\nMix.install(\n  [\n    {:grammar, path: Path.join(__DIR__, \"..\"), env: :dev}\n  ],\n  lockfile: :grammar\n)\n```","ref":"grammar_intro.html#local-setup"},{"type":"extras","title":"Grammar with a dedicated callback module - Grammar to parse a comma separated list of numbers","doc":"```elixir\ndefmodule CB do\n  def list([\"[\", nil, \"]\"]), do: []\n  def list([\"[\", elements, \"]\"]), do: elements\n\n  def elements?([element, nil]), do: [element]\n  def elements?([element, tail]), do: [element | tail]\n\n  def elements_tail?([\",\", element, nil]), do: [element]\n  def elements_tail?([\",\", element, tail]), do: [element | tail]\n\n  def element([number]), do: number\n\n  def number([number_str]), do: String.to_integer(number_str)\nend\n\ngrammar =\n  Grammar.new()\n  |> Grammar.add_clause(:list, [\"[\", :elements?, \"]\"], &CB.list/1)\n  |> Grammar.add_clause(:elements?, [:element, :elements_tail?], &CB.elements?/1, true)\n  |> Grammar.add_clause(:elements_tail?, [\",\", :element, :elements_tail?], &CB.elements_tail?/1, true)\n  |> Grammar.add_clause(:element, [:number], &CB.element/1)\n  |> Grammar.add_clause(:number, [~r/[0-9]+/], &CB.number/1)\n  |> Grammar.prepare!()\n  |> Grammar.start(:list)\n```","ref":"grammar_intro.html#grammar-with-a-dedicated-callback-module"},{"type":"extras","title":"Parse some valid inputs - Grammar to parse a comma separated list of numbers","doc":"```elixir\n# Empty list\nGrammar.loop(grammar, Grammar.Tokenizer.new(\"[]\"))\n```\n\n```elixir\n# Single item list\nGrammar.loop(grammar, Grammar.Tokenizer.new(\"[12]\"))\n```\n\n```elixir\n# Regular list\nGrammar.loop(grammar, Grammar.Tokenizer.new(\"[42, 53, 453]\"))\n```","ref":"grammar_intro.html#parse-some-valid-inputs"},{"type":"extras","title":"Try parse invalid inputs - Grammar to parse a comma separated list of numbers","doc":"```elixir\nGrammar.loop(grammar, Grammar.Tokenizer.new(\"ident\"))\n```\n\n```elixir\nGrammar.loop(grammar, Grammar.Tokenizer.new(\"[12,\"))\n```\n\n```elixir\nGrammar.loop(grammar, Grammar.Tokenizer.new(\"[12,]\"))\n```","ref":"grammar_intro.html#try-parse-invalid-inputs"},{"type":"extras","title":"Changing the start rule - Grammar to parse a comma separated list of numbers","doc":"Changing the starting rule allows for testing a sub-part of a grammar.\n\n<!-- livebook:{\"break_markdown\":true} -->","ref":"grammar_intro.html#changing-the-start-rule"},{"type":"extras","title":"Change `grammar` to start at rule `:element` - Grammar to parse a comma separated list of numbers","doc":"Only a number can be extracted from the input\n\n```elixir\ngrammar = Grammar.start(grammar, :element)\nGrammar.loop(grammar, Grammar.Tokenizer.new(\"12\"))\n```\n\nTrying to reading anything else will fail\n\n```elixir\nGrammar.loop(grammar, Grammar.Tokenizer.new(\"[12]\"))\n```","ref":"grammar_intro.html#change-grammar-to-start-at-rule-element"},{"type":"extras","title":"Change `grammar` to start at rule `:elements_tail?` - Grammar to parse a comma separated list of numbers","doc":"The parser can now extract \"nothing\" (as `:elements_tail?` is an `epsilon` rule), or a a list of `,  `.\n\n```elixir\ngrammar = Grammar.start(grammar, :elements_tail?)\nGrammar.loop(grammar, Grammar.Tokenizer.new(\"\"))\n```\n\n```elixir\nGrammar.loop(grammar, Grammar.Tokenizer.new(\", 122, 134\"))\n```\n\nAgain, trying to reading anything else will fail\n\n```elixir\nGrammar.loop(grammar, Grammar.Tokenizer.new(\"122, 134\"))\n```","ref":"grammar_intro.html#change-grammar-to-start-at-rule-elements_tail"},{"type":"extras","title":"Musing with CodeGen","doc":"# Musing with CodeGen\n\n```elixir\nMix.install([:grammar])\n```","ref":"musing_with_codegen.html"},{"type":"extras","title":"Local setup - Musing with CodeGen","doc":"To use this livebook within a local clone the project, use the following setup, and launch the livebook from it's directory.\n\n<!-- livebook:{\"force_markdown\":true} -->\n\n```elixir\nMix.install(\n  [\n    {:grammar, path: Path.join(__DIR__, \"..\"), env: :dev}\n  ],\n  lockfile: :grammar\n)\n```","ref":"musing_with_codegen.html#local-setup"},{"type":"extras","title":"Defining a grammar with CodeGen.Clause - Musing with CodeGen","doc":"Let's define `target_module_name` as holding the name of the module we want to define as a specialized parser module.\n\nThis binding will be used all along this livebook when it comes to refer to the module name.\n\n```elixir\ntarget_module_name = Pouet\n```\n\nNow define the clauses that will constitute the grammar.\n\n```elixir\nalias Grammar.CodeGen\n\ngrammar_clauses = [\n  Grammar.CodeGen.Clause.new(:foo_bar, quote do [:foo] end, Code.string_to_quoted!(~S/[what] = params; \"Found foo #{what} \" <> <<0x1F92A::utf8>>/), false),\n  Grammar.CodeGen.Clause.new(:foo_bar, quote do [:bar] end, Code.string_to_quoted!(~S/[what] = params; \"Found bar #{what} \" <> <<0x1F92A::utf8>>/), false),\n\n  # Here we use Code.string_to_quoted!/1 to ensure the context in quotes is `nil` as it would be during compilation.\n  # Simply quoting would result in quoted expression in the context of `Elixir` as we are here evaluating the code (and not compiling it).\n  # \n  # Another option in our current context of evaluating things, is to explicitly state that params should be NOT hygienized\n  # e.g. Grammar.CodeGen.Clause.new(:foo, quote do [\"foo\"] end, quote do [foo] = var!(params); \"#{foo} \" <> <<0x1F600::utf8>> <> \" !\" end, false)\n  Grammar.CodeGen.Clause.new(:foo, quote do [\"foo\"] end, Code.string_to_quoted!(~S/[foo] = params; \"#{foo} \" <> <<0x1F601::utf8>> <> \" !\"/), false),\n\n  Grammar.CodeGen.Clause.new(:bar, quote do [\"bar\"] end, Code.string_to_quoted!(~S/[bar] = params; \"#{bar} \" <> <<0x1F601::utf8>> <> \" !\"/), false)\n]\n\n```","ref":"musing_with_codegen.html#defining-a-grammar-with-codegen-clause"},{"type":"extras","title":"Generating and using a grammar - Musing with CodeGen","doc":"```elixir\ngrammar_definition_ast = Grammar.CodeGen.build_grammar(target_module_name, grammar_clauses)\n```","ref":"musing_with_codegen.html#generating-and-using-a-grammar"},{"type":"extras","title":"output the generated code in its Elixir form - Musing with CodeGen","doc":"```elixir\ngrammar_definition_ast\n|> Macro.to_string()\n|> IO.puts()\n```","ref":"musing_with_codegen.html#output-the-generated-code-in-its-elixir-form"},{"type":"extras","title":"Manually define the callback module that fits the grammar and make some tests - Musing with CodeGen","doc":"```elixir\nquote do\n  defmodule unquote(target_module_name) do\n    def clause_0_foo_bar([what]), do: \"foo #{what}\"\n    def clause_1_foo_bar([what]), do: \"bar #{what}\"\n    def clause_0_foo([\"foo\"]), do: \"foo\"\n    def clause_0_bar([\"bar\"]), do: \"bar\"\n  end\nend\n|> Code.eval_quoted()\n\n{grammar, _bindings} = Code.eval_quoted(grammar_definition_ast)\n\nGrammar.loop(grammar, Grammar.Tokenizer.new(\"foo\")) |> IO.inspect()\nGrammar.loop(grammar, Grammar.Tokenizer.new(\"bar\")) |> IO.inspect()\n```\n\nThe module we just define manually does NOT export the `parse/1` function unlike the generated modules.\n\n```elixir\ntry do\n  target_module_name.parse(\"coucou\")\nrescue\n  e in UndefinedFunctionError -> e\nend\n```","ref":"musing_with_codegen.html#manually-define-the-callback-module-that-fits-the-grammar-and-make-some-tests"},{"type":"extras","title":"Use CodeGen to produce the callback module - Musing with CodeGen","doc":"","ref":"musing_with_codegen.html#use-codegen-to-produce-the-callback-module"},{"type":"extras","title":"Generate the code of the callback functions refered to in the grammar clauses. - Musing with CodeGen","doc":"```elixir\ncallback_functions_ast = Grammar.CodeGen.build_rule_body_functions(grammar_clauses)\n```","ref":"musing_with_codegen.html#generate-the-code-of-the-callback-functions-refered-to-in-the-grammar-clauses"},{"type":"extras","title":"output the generated code in its Elixir form - Musing with CodeGen","doc":"```elixir\ncallback_functions_ast\n|> Macro.to_string()\n|> IO.puts()\n```\n\n### (re)Define the callback module using generated function bodies\n\n```elixir\n{{:module, module_name, code, _exported_funs}, _bindings} = \nquote do\n  defmodule unquote(target_module_name) do\n    # Use var!/1 to avoid the compiler sanitize `code`\n    # as its value in passed through bindings\n    @grammar unquote(grammar_definition_ast)\n    \n    unquote_splicing(callback_functions_ast)\n\n    def parse(input) do\n      tokenizer = Grammar.Tokenizer.new(input)\n      Grammar.loop(@grammar, tokenizer)\n    end\n  end\nend\n|> Code.eval_quoted()\n```\n\n```elixir\nmodule_name.parse(\"foo\")\n```\n\n```elixir\nmodule_name.parse(\"bar\")\n```","ref":"musing_with_codegen.html#output-the-generated-code-in-its-elixir-form"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}